from unittest.mock import patch

import pytest

from gso.schedules.scheduling import scheduler


@pytest.fixture
def mock_celery():
    with patch("gso.schedules.scheduling.current_app") as mock_app:
        yield mock_app


def test_scheduler_updates_beat_schedule(mock_celery):
    mock_celery.conf.beat_schedule = {}

    @scheduler(name="A cool task", minute="0", hour="0", day_of_week="*", day_of_month="*", month_of_year="*")
    def mock_task():
        return "task result"

    assert "mock_task" in mock_celery.conf.beat_schedule
    scheduled = mock_celery.conf.beat_schedule["mock_task"]
    assert scheduled["schedule"].minute == {0}
    assert scheduled["schedule"].hour == {0}
    assert scheduled["task"] == "test.schedules.test_scheduling.mock_task"
    assert scheduled["name"] == "A cool task"


def test_scheduled_task_still_works():
    """Ensure that the scheduler decorator does not change the behavior of the function it decorates."""

    @scheduler(name="A cool task", minute="0", hour="0", day_of_week="*", day_of_month="*", month_of_year="*")
    def mock_task():
        return "task result"

    result = mock_task()
    assert result == "task result"
